from django.db import models
from django.core.urlresolvers import reverse
from django_extensions.db.fields import AutoSlugField
import mptt

from urlparse import urljoin

class Category(models.Model):
    parent = models.ForeignKey('self',
                               null=True,
                               blank=True,
                               related_name='children')
    name = models.CharField(max_length=50)
    slug = AutoSlugField(max_length=50,
                         overwrite=True,
                         populate_from='name')
    url = models.TextField(editable=False)

    class Meta:
        verbose_name_plural = "categories"
        unique_together = (("name", "slug", "parent"), )
        ordering = ("tree_id", "lft")

    def __unicode__(self):
        return self.url

    def save(self, force_insert=False, force_update=False, **kwargs):
        super(Category, self).save(
            force_insert=force_insert,
            force_update=force_update,
            **kwargs)
        self.update_url()

    def get_tree(self, *args):
        """
        Return the tree structure for this element
        """
        level_representation = "--"
        if self.level == 0:
            node = "| "
        else:
            node = "+ "
        _tree_structure = node + level_representation * self.level
        return _tree_structure
    get_tree.short_description = 'tree'

    def get_repr(self, *args):
        """
        Return the branch representation for this element
        """
        level_representation = "--"
        if self.level == 0:
            node = "| "
        else:
            node = "+ "
        _tree_structure = node + level_representation * self.level + ' ' + self.name
        return _tree_structure
    get_repr.short_description = 'representation'

    def tree_order(self):
        return str(self.tree_id) + str(self.lft)

    def update_url(self):
        """
        Updates the url for this Category and all children Categories.
        """
        url = urljoin(getattr(self.parent, 'url', '') + '/', self.slug)
        if url != self.url:
            self.url = url
            self.save()

            for child in self.get_children():
                child.update_url()

mptt.register(Category, order_insertion_by=['name'])

class Selection (models.Model):
    name = models.CharField(max_length=50)
    categories = models.ManyToManyField(Category)

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('selection', args=[self.pk])



from django import forms
from apps.categories.models import Category, Selection
from fancytree.widgets import FancyTreeWidget

categories = Category.objects.order_by('tree_id', 'lft')

class SelectionForm(forms.ModelForm):
    class Meta:
        model = Selection
        fields = ('name', 'categories')
        widgets = {
            'categories': FancyTreeWidget(queryset=categories)
        }



from django.shortcuts import render, get_object_or_404, redirect
from apps.categories.forms import SelectionForm
from apps.categories.models import Selection


def home(request):
    form = SelectionForm(request.POST or None)
    if request.method == "POST":
        if form.is_valid():
            selection = form.save()
            return redirect('selection', pk=selection.pk)
    selections = Selection.objects.all()
    return render(request, "home.html", {'form': form, 'selections': selections})

def selection(request, pk):
    selection = get_object_or_404(Selection, pk=pk)
    if request.method == "POST":
        form = SelectionForm(request.POST, instance=selection)
        if form.is_valid():
            selection = form.save()
            return redirect('selection', pk=selection.pk)
    else:
        form = SelectionForm(instance=selection)
    selections = Selection.objects.all()
    return render(request, "home.html", {'form': form, 'selections': selections})



from django.conf.urls import patterns, include, url

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$', 'views.home', name='home'),
    url(r'^selection/(?P<pk>\d)$', 'views.selection', name='selection'),

    url(r'^admin/', include(admin.site.urls)),
)

from django.conf import settings
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

urlpatterns += staticfiles_urlpatterns()

if settings.DEBUG:
    urlpatterns += patterns('', (
        r'^site_media/media/(?P<path>.*)$',
        'django.views.static.serve',
        {'document_root': settings.MEDIA_ROOT}
        ),
    )



from django.contrib import admin

from apps.categories.models import Category

class CategoryAdminOption(admin.ModelAdmin):
    list_display = ("get_repr", "get_tree")
    ordering = ("url",)
    search_fields = ('name', )


admin.site.register(Category, CategoryAdminOption)



#!/usr/bin/env python

from setuptools import setup, find_packages
import fancytree
import os

setup(name='django-fancytree',
      version=fancytree.__version__,
      description='Django forms widget that uses Fancytree to display tree data',
      author='Riccardo Magliocchetti',
      author_email='riccardo.magliocchetti@gmail.com',
      url='https://github.com/xrmx/django-fancytree',
      packages=find_packages(),
      keywords=['django', 'fancytree', 'mptt', 'tree'],
      classifiers=[
          'Programming Language :: Python',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Topic :: Software Development :: Libraries :: Application Frameworks',
      ],
      long_description=open(
          os.path.join(os.path.dirname(__file__), 'README.rst'),
      ).read().strip(),
      install_requires=[
          'Django',
          'django-mptt',
      ],
      include_package_data=True
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('slug', django_extensions.db.fields.AutoSlugField(editable=False, populate_from=b'name', blank=True, overwrite=True)),
                ('url', models.TextField(editable=False)),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('parent', models.ForeignKey(related_name='children', blank=True, to='categories.Category', null=True)),
            ],
            options={
                'ordering': ('tree_id', 'lft'),
                'verbose_name_plural': 'categories',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Selection',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('categories', models.ManyToManyField(to='categories.Category')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='category',
            unique_together=set([('name', 'slug', 'parent')]),
        ),
    ]
